# mandayejs - saml reverse proxy
# Copyright (C) 2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from importlib import import_module

from django.conf import settings
from django.conf.urls import url
from django.http import Http404
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import resolve
from django.utils.text import slugify


def get_app_settings():
    module_name, app_settings = tuple(settings.SITE_APP.rsplit('.', 1))
    module = import_module(module_name)
    return getattr(module, app_settings)()


def app_web_services(request, path):
    app = get_app_settings()
    if hasattr(app, 'SITE_WEBSERVICES'):
        view, args, kwargs = resolve(request.path, urlconf=app.SITE_WEBSERVICES)
        return view(request, *args, **kwargs)
    raise Http404


REQUIRED_PARAMS = [
    'SITE_LOGIN_PATH',
    'SITE_LOCATORS',
    'SITE_AUTH_CHECKER',
    'SITE_AUTH_COOKIE_KEYS',
]


class Param(object):

    def __init__(self, initval=None, name=None):
        self.value = initval
        if name:
            self.name = name

    def __set__(self, instance, value):
        self.value = value

    def __get__(self, instance, owner):
        if instance is None:
            return self
        if hasattr(settings, self.name):
            return getattr(settings, self.name)
        return self.value


class LoginPathParam(Param):
    name = 'SITE_LOGIN_PATH'


class LocatorsParam(Param):
    name = 'SITE_LOCATORS'


class AuthCheckerParam(Param):
    name = 'SITE_AUTH_CHECKER'


class AuthCookieKeysParam(Param):
    name = 'SITE_AUTH_COOKIE_KEYS'


class ForceRedirctUrlParam(Param):
    name = 'SITE_FORCE_REDIRECT_URL'


class ForceRedirectLocatorParam(Param):
    name = 'SITE_FORCE_REDIRECT_LOCATOR'


class FormSubmitElementParam(Param):
    name = 'SITE_FORM_SUBMIT_ELEMENT'


class LogoutLocatorParam(Param):
    name = 'LOGOUT_LOCATOR'


class AppScriptParam(Param):
    name = 'APP_SCRIPT'


class AppSettings(object):

    SITE_FORM_SUBMIT_ELEMENT = FormSubmitElementParam('input[type=submit], button')

    def __init__(self, *args, **kwargs):
        for param in REQUIRED_PARAMS:
            if not hasattr(self, param):
                raise ImproperlyConfigured('missing required parameter <%s>' % param)

    @property
    def name(self):
        if hasattr(settings, 'SITE_APP_NAME'):
            return settings.SITE_APP_NAME
        return self.__class__.__name__

    @property
    def slug(self):
        return slugify(self.__class__.__name__)


class AppWebservice(object):
    '''- endpoints = [
        {'foo': 'https://example.org/foo/'},
        {'bar': 'https://example.org/bar/'},
    ]
    - urlpatterns = [
        url('user/(?P<username>\w+)/$', views.myapp.user_details, name='user-details'),
        url('user/(?P<username>\w+/books/$)', views.myapp.user_books, name='user-books'),
    ]
    '''

    def __init__(self, endpoints=None, urlpatterns=None):
        self.endpoints = endpoints
        self.urlpatterns = urlpatterns

    def get_endpoint(self, name):
        for endpoint in self.endpoints:
            if endpoint.get(name, None):
                return endpoint[name]
        return None


# Test App Settings
class Test(AppSettings):
    SITE_LOGIN_PATH = LoginPathParam('/')
    SITE_LOCATORS = LocatorsParam([
        {
            'id': '#login',
            'label': 'Login',
            'name': 'login',
            'kind': 'string',
        },
        {
            'id': '#password',
            'label': 'Password',
            'name': 'password',
            'kind': 'password'
        }
    ])
    SITE_AUTH_CHECKER = AuthCheckerParam('js/test/auth.checker.js')
    SITE_AUTH_COOKIE_KEYS = AuthCookieKeysParam(['test'])
    SITE_FORCE_REDIRECT_URL = ForceRedirctUrlParam('/whatever')


# Duonet App Settings
class Duonet(AppSettings):
    SITE_LOGIN_PATH = LoginPathParam('/')
    SITE_LOCATORS = LocatorsParam([
        {
            'id': '#txtNomFoyer',
            'label': 'Nom de famille',
            'name': 'txtNomFoyer',
            'kind': 'string',
            'required': True,
            'help': '',
        },
        {
            'id': '#txtDateNaissance',
            'label': 'Date de naissance',
            'name': 'txtDateNaissance',
            'kind': 'date',
            'required': True,
            'help': 'exemple 16/06/2008'
        },
        {
            'id': '#txtCode',
            'label': 'Mot de passe',
            'name': 'txtCode',
            'kind': 'password',
            'required': True,
            'help': ''
        },
    ])
    SITE_AUTH_CHECKER = AuthCheckerParam('duonet/js/auth.checker.js')
    SITE_AUTH_COOKIE_KEYS = AuthCookieKeysParam([
        'ASP.NET_SessionId',
    ])
    SITE_APP_SCRIPTS = AppScriptParam([
        'duonet/js/duonet.js',
    ])

    SITE_FORCE_REDIRECT_URL = ForceRedirctUrlParam('/Connect.aspx')

    SITE_FORM_SUBMIT_ELEMENT = FormSubmitElementParam('input[type=submit]')

    SITE_LOGOUT_LOCATOR = LogoutLocatorParam('#lnkDisconnect')


# Archimed App Settings
class Archimed(AppSettings):
    SITE_LOGIN_PATH = LoginPathParam('/')
    SITE_LOCATORS = LocatorsParam([
        {
            'id': '#carte',
            'label': 'Identifiant',
            'name': 'carte',
            'kind': 'string',
            'required': True,
            'help': '',
        },
        {
            'id': '#code',
            'label': 'Mot de passe',
            'name': 'code',
            'kind': 'password',
            'required': True,
            'help': ''
        },
    ])
    SITE_AUTH_CHECKER = AuthCheckerParam('archimed/js/auth.checker.js')
    SITE_AUTH_COOKIE_KEYS = AuthCookieKeysParam([
        'S_ARCHIMED_CRYSTAL_AUTH'
    ])

    SITE_APP_SCRIPTS = AppScriptParam([
        'archimed/js/archimed.js'
    ])

    SITE_FORCE_REDIRECT_LOCATOR = ForceRedirectLocatorParam('.connectBox')

    SITE_WEBSERVICES = AppWebservice(
        [
            {'account_details': '/DEFAULT/Ermes/Services/ILSClient.svc/RetrieveAccount'},
            {'login_url': '/DEFAULT/Ermes/Recherche/logon.svc/logon'}
        ],
        [
            url(r'account/(?P<username>\w+)/$', 'mandayejs.applications.views.archimed_account_details',
                name='archimed-account-details')
        ]
    )

    SITE_LOGOUT_LOCATOR = LogoutLocatorParam('.account_logoff')


# Arpege App Settings
class Arpege(AppSettings):
    SITE_LOGIN_PATH = LoginPathParam('/index.do')
    SITE_LOCATORS = LocatorsParam([
        {
            'id': '#cdfmll',
            'label': 'Code famille',
            'name': 'cdfmll',
            'kind': 'string',
            'required': True,
            'help': '',
        },
        {
            'id': '#mtdpss',
            'label': 'Mot de passe',
            'name': 'mtdpss',
            'kind': 'password',
            'required': True,
            'help': '',
        }
    ])
    SITE_AUTH_CHECKER = AuthCheckerParam('arpege/js/auth.checker.js')
    SITE_AUTH_COOKIE_KEYS = AuthCookieKeysParam([
        'JSESSIONID',
    ])

    SITE_APP_SCRIPTS = AppScriptParam([
        'arpege/js/arpege.js',
    ])

    SITE_FORCE_REDIRECT_LOCATOR = ForceRedirectLocatorParam(r'.formulaire')

    SITE_LOGOUT_LOCATOR = LogoutLocatorParam('#espace-login form input[type=submit]')


class Imuse(AppSettings):
    SITE_LOGIN_PATH = LoginPathParam('extranet/login/gen_index_groupe.php?nav=autre')

    SITE_LOCATORS = LocatorsParam([
        {
            'id': '#INDEX_USER_ID',
            'label': 'Identifiant',
            'name': 'INDEX_USER_ID',
            'kind': 'string',
            'required': True,
            'help': ''
        },
        {
            'id': '#INDEX_USER_PWD',
            'label': 'Mot de passe',
            'name': 'INDEX_USER_PWD',
            'kind': 'password',
            'required': True,
            'help': ''
        }
    ])

    SITE_AUTH_CHECKER = AuthCheckerParam('imuse/js/auth.checker.js')

    SITE_AUTH_COOKIE_KEYS = AuthCookieKeysParam([
        'iMuse-extranet'
    ])

    SITE_FORCE_REDIRECT_LOCATOR = ForceRedirectLocatorParam('#INDEX_TBL_LOGIN')

    SITE_FORM_SUBMIT_ELEMENT = FormSubmitElementParam('#INDEX_BT_LOGIN')


class Sezhame(AppSettings):
    SITE_LOGIN_PATH = LoginPathParam('/sezhame/page/connexion-abonne?destination=user')

    SITE_LOCATORS = LocatorsParam([
        {
            'id': '#edit-user',
            'label': 'Numero de cqrte',
            'name': 'edit-user',
            'kind': 'string',
            'required': True,
            'help': ''
        },
        {
            'id': '#edit-password',
            'label': 'Mot de passe',
            'name': 'edit-password',
            'kind': 'password',
            'required': True,
            'help': ''
        }
    ])

    SITE_AUTH_CHECKER = AuthCheckerParam('sezhame/js/auth.checker.js')

    SITE_AUTH_COOKIE_KEYS = AuthCookieKeysParam([
        'SESSf36da25307ad6240a58ddd4f4b138952',
        'ASPSESSIONIDQSDRASTR'
    ])

    SITE_FORCE_REDIRECT_LOCATOR = ForceRedirectLocatorParam('#dk-opac15-login-form')


class Teamnet(AppSettings):
    SITE_LOGIN_PATH = LoginPathParam('/auth/teamnetauth')

    SITE_LOCATORS = LocatorsParam([
        {
            'id': '#login',
            'label': 'Mon identifiant',
            'name': 'login',
            'kind': 'string',
            'required': True,
            'help': ''
        },
        {
            'id': '#mdp',
            'label': 'Mon mot de passe',
            'name': 'mdp',
            'kind': 'password',
            'required': True,
            'help': ''
        }
    ])

    SITE_FORM_SUBMIT_ELEMENT = FormSubmitElementParam("input[type=submit][value='Me connecter']")

    SITE_AUTH_CHECKER = AuthCheckerParam('teamnet/js/auth.checker.js')

    SITE_AUTH_COOKIE_KEYS = AuthCookieKeysParam([
        'JSESSIONID',
    ])

    SITE_FORCE_REDIRECT_LOCATOR = ForceRedirectLocatorParam('#loginForm')

    SITE_LOGOUT_LOCATOR = LogoutLocatorParam(".infoUtilisateur[alt=Deconnexion]")
