from __future__ import absolute_import

import os
import csv
import json
import ldif
import logging
from optparse import make_option

from django.core.management.base import BaseCommand, CommandError
from django.db import IntegrityError
from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured

from mandayejs.mandaye.models import UserCredentials
from mellon.models import UserSAMLIdentifier
from mellon.utils import get_idps


logger = logging.getLogger(__name__)


def get_issuer():
    idps = list(get_idps())
    if not idps:
        raise ImproperlyConfigured('ENTITY_ID or METADATA_URL required in settings.MELLON_IDENTITY_PROVIDERS')
    idp = idps[0]
    issuer = idp.get('METADATA_URL', None) or idp.get('ENTITY_ID', None)
    return issuer


class Command(BaseCommand):
    args = '<filename>'
    help = 'Migrate users from ldif file or csv file'

    option_list = BaseCommand.option_list +(
        make_option(
            '--ldap',
            action='store_true',
            default=False,
            help='Migrate users from a ldap dump file'
        ),
        make_option(
            '--csv',
            action='store_true',
            default=False,
            help='Migrate users from a csv file'
        )
    )

    def handle(self, *args, **kwargs):
        if len(args) < 1:
            self.stdout.write('An input file is required')
            return False

        if kwargs.get('csv'):
            for filename in args:
                data = self.get_csv_data(filename)
                self.migrate(data)
        else:
            for filename in args:
                data = self.get_ldif_data(filename)
                data = [ d[1] for d in data]
                data = [
                    { k : ''.join(v) for k,v in d.items()} for d in data
                ]
                self.migrate(data)

    def get_ldif_data(self, filename):
        with open(filename, 'rb') as fd:
            return ldif.ParseLDIF(fd)

    def get_csv_data(self, filename):
        with open(filename, 'rb') as fd:
            fieldnames = ['idpUniqueID', 'spPostValues']
            reader = csv.DictReader(fd, delimiter=';', quotechar='|', fieldnames=fieldnames)
            return list(reader)


    def migrate(self, parsed_data):
        issuer = get_issuer()

        for data in parsed_data:
            try:
                name_id = data.get('idpUniqueID')
                credentials = json.loads(data.get('spPostValues'))

                user, created = User.objects.get_or_create(username=name_id,
                        last_name=data.get('spLogin', ''))

                uc = UserCredentials(user=user, locators=credentials)
                uc.save()

                saml_id, created = UserSAMLIdentifier.objects.get_or_create(user=user, name_id=name_id, issuer=issuer)

                logger.debug('{idpUniqueID} imported'.format(**data))
            except (IntegrityError,) as e:
                logger.debug(e)
                continue
